using System.Collections.Generic;
using UnityEngine;
using SimpleJSON;
using System.Collections;
using UnityEngine.Events;
using UnityEngine.UI;

/* 95% written by MeshedVR
Doc'0c adapted it to a plugin
*/
namespace ReVAMped
{

    public class WindControl : MVRScript
    {
        Atom windDirection;
        public static Vector3 globalWind = Vector3.zero;
        protected static Vector3  newDirection = Vector3.zero;
        protected string _missingReceiverStoreId = string.Empty;
        public bool isGlobal;
		public bool isAvatar;
        public bool isRadial;
        public bool isBlowing;
        public bool oscillation=false;
        protected float windRandomX;
        protected float windRandomY;
        protected float windRandomZ;
        protected JSONStorableBool isGlobalJSON;
		protected JSONStorableBool isAvatarJSON;
        protected JSONStorableBool isBlowingJSON;
        protected Atom receivingAtom;
        protected JSONStorableStringChooser atomJSON;
        protected JSONStorable receiver;
        protected JSONStorableStringChooser receiverJSON;
        protected string _receiverTargetName;
        protected JSONStorableVector3 receiverTarget;
        protected JSONStorableStringChooser receiverTargetJSON;
        protected JSONStorableFloat currentMagnitudeJSON;
        protected JSONStorableBool autoJSON;
        protected JSONStorableBool oscillatingJSON;
        protected JSONStorableFloat periodJSON;
        protected JSONStorableFloat quicknessJSON;
        protected JSONStorableFloat windDecay;
        protected JSONStorableFloat avatarMobility;
        protected JSONStorableFloat windStrength;
        protected JSONStorableFloat windTurbulence;
        protected JSONStorableFloat targetMagnitudeJSON;
        protected JSONStorableFloat lowerMagnitudeJSON;
        protected JSONStorableFloat lowerMagnitudeRatioJSON;
        protected JSONStorableFloat quicknessRatioJSON;
        protected JSONStorableFloat oscillationVariance;
        protected JSONStorableFloat upperMagnitudeJSON;
        protected float timer;
        protected float loadTimer;
        protected Vector3 direction;
        protected Dictionary<string, ForceReceiver> receiverNameToForceReceiver;
        protected Rigidbody RB;
        protected UIDynamicButton btn;
        protected string _lastBrowseDir;
        protected bool loaded;

        protected JSONStorableFloat mobility_strong;
        protected JSONStorableFloat mobility_medium;
        protected JSONStorableFloat mobility_light;
        protected JSONStorableFloat mobility_controller;

        protected JSONStorableBool isRadialJSON;

        protected Dictionary<string, bool> RB_enabled;
        protected Dictionary<string, JSONStorableFloat> RB_mobility;
        protected List<string> excludeAtom;

        protected void SyncReceiverRB(string receiver)
        {
            if (receiver != null)
            {
                ForceReceiver fr;
                if (receiverNameToForceReceiver.TryGetValue(receiver, out fr))
                {
                    RB = fr.GetComponent<Rigidbody>();
                }
                else
                {
                    RB = null;
                }
            }
            else
            {
                RB = null;
            }
        }

        protected void initRBdicts()
        {
            loaded=false;
            loadTimer=100f;
            RB_enabled = new Dictionary<string, bool>();
            RB_mobility = new Dictionary<string, JSONStorableFloat>();
            excludeAtom= new List<string>();

            excludeAtom.Add("Person");
            excludeAtom.Add("Light");
            excludeAtom.Add("Player");
            excludeAtom.Add("Camera");
            excludeAtom.Add("Core");
            excludeAtom.Add("Sound");
            excludeAtom.Add("Empty");
            excludeAtom.Add("Animation");
            excludeAtom.Add("Force");
            excludeAtom.Add("Web");
            excludeAtom.Add("UI");

            mobility_light=new JSONStorableFloat("mobility_light", 1f, 0.1f, 1.0f, true, true);
            mobility_strong=new JSONStorableFloat("mobility_strong", 0.1f, 0.1f, 1.0f, true, true);
            mobility_medium=new JSONStorableFloat("mobility_medium", 0.5f, 0.1f, 1.0f, true, true);
            mobility_controller=new JSONStorableFloat("mobility_controller", 0.1f, 0.1f, 1.0f, true, true);
        }

        protected void debug_print_dictionary()
        {
            foreach(KeyValuePair<string,bool> RB_elem in RB_enabled){
                SuperController.LogMessage("updated RB:"+RB_elem.Key+" val:"+RB_enabled[RB_elem.Key]+"");
            }
        }

        protected void debug_print_allRB()
        {

        }
        
        protected void SyncIsGlobal(bool b)
        {
            isGlobal = b;
        }
		
		protected void SyncIsAvatar(bool b)
        {
            isAvatar = b;
        }

        protected void SyncIsBlowing(bool b)
        {
            isBlowing = b;
        }

        protected void SyncIsRadial(bool b)
        {
            isRadial = b;
        }

        protected void getAtomList()
        {
            foreach (string atomUiD in SuperController.singleton.GetAtomUIDs())
            {
                var receivingAtom = SuperController.singleton.GetAtomByUid(atomUiD);
                string windTarget="";
                //SuperController.LogMessage("Atom:"+atomUiD+"");
                foreach (string elem in receivingAtom.GetStorableIDs())
                {
                    if(elem.Contains("WindControl_Target"))
                    {
                        windTarget=elem;
                    }
                }
                
                if(windTarget!="")
                {
                    RB_enabled.Add(atomUiD,SuperController.singleton.GetAtomByUid(atomUiD).GetStorableByID(windTarget).GetBoolJSONParam("Wind Enabled").val);
                    RB_mobility.Add(atomUiD,SuperController.singleton.GetAtomByUid(atomUiD).GetStorableByID(windTarget).GetFloatJSONParam("Mobility"));
                }
                
                /*
                foreach (ForceReceiver fr in containingAtom.forceReceivers)
                {
                    if(fr.name=="object")
                    {
                        RB_enabled.Add(atomUiD,false);
                        RB_mobility.Add(atomUiD,mobility_light);
                    }
                }
                */
                
            } 
        }

        protected void SyncAtom(string atomUID)
        {
            List<string> stringList = new List<string>();
            stringList.Add("None");
            if (atomUID != null)
            {
                receivingAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (receivingAtom != null)
                {
                    foreach (string storableId in receivingAtom.GetStorableIDs())
                    {
                        stringList.Add(storableId);
                    }
                    receiverNameToForceReceiver = new Dictionary<string, ForceReceiver>();
                    foreach (ForceReceiver fr in receivingAtom.forceReceivers)
                    {
                        receiverNameToForceReceiver.Add(fr.name, fr);
                    }
                }
            }
            else
                receivingAtom = (Atom)null;

            receiverJSON.choices = stringList;
            receiverJSON.val = "None";
        }

        protected void CheckMissingReceiver()
        {
            if (!(_missingReceiverStoreId != string.Empty)
                || !(receivingAtom != null)
                || !(receivingAtom.GetStorableByID(_missingReceiverStoreId) != null))
                return;

            string receiverTargetName = _receiverTargetName;
            SyncReceiver(_missingReceiverStoreId);
            _missingReceiverStoreId = string.Empty;
            insideRestore = true;
            receiverTargetJSON.val = receiverTargetName;
            insideRestore = false;
        }

        protected void SyncReceiver(string receiverID)
        {
            List<string> stringList = new List<string>();
            stringList.Add("None");
            if (receivingAtom != null && receiverID != null)
            {
                receiver = receivingAtom.GetStorableByID(receiverID);
                if (receiver != null)
                {
                    foreach (string vector3ParamName in receiver.GetVector3ParamNames())
                    {
                        stringList.Add(vector3ParamName);
                    }
                }
                else if (receiverID != "None")
                    _missingReceiverStoreId = receiverID;
            }
            else
            {
                receiver = null;
            }

            receiverTargetJSON.choices = stringList;
            receiverTargetJSON.val = "None";
        }

        protected void SyncReceiverTarget(string receiverTargetName)
        {
            _receiverTargetName = receiverTargetName;
            receiverTarget = (JSONStorableVector3)null;
            if (receiver != null && receiverTargetName != null)
                receiverTarget = receiver.GetVector3JSONParam(receiverTargetName);
        }

        public UIDynamicTextField CreateTextInput(JSONStorableString jss, bool rightSide = false)
        {
            var textfield = CreateTextField(jss, rightSide);
            textfield.height = 1f;
            //textfield.backgroundColor = Color.white;
            var input = textfield.gameObject.AddComponent<InputField>();
            var rect = input.GetComponent<RectTransform>().sizeDelta = new Vector2(1f, 0.4f);
            input.textComponent = textfield.UItext;
            jss.inputField = input;
            return textfield;
        }


        public override void Init()
        {
            try
            {
                //loadTimer=10f;
                initRBdicts();
                //updateObjectList();
                
                
                                
                //isGlobalJSON = new JSONStorableBool("Affect Cloth/Hair", false, new JSONStorableBool.SetBoolCallback(SyncIsGlobal));
				isAvatarJSON = new JSONStorableBool("Affect Objects", false, new JSONStorableBool.SetBoolCallback(SyncIsAvatar));



                receiverTargetJSON = new JSONStorableStringChooser(
                  "receiverTarget", (List<string>)null, (string)null,
                  "Target", new JSONStorableStringChooser.SetStringCallback(SyncReceiverTarget));

                currentMagnitudeJSON = new JSONStorableFloat("Current Magnitude (view)", 0.0f, -50f, 50f, false, true);
                autoJSON = new JSONStorableBool("Randomize Current Magnitude", true);
                oscillatingJSON = new JSONStorableBool("Oscillate Current Magnitude", false);
                periodJSON = new JSONStorableFloat("period", 0.03f, 0.0f, 1f, false, true);
                quicknessJSON = new JSONStorableFloat("Quickness (View)", 30f, 0.0f, 100f, true, true);
                lowerMagnitudeJSON = new JSONStorableFloat("Lower Magnitude (View)", -50f, -50f, 200f, false, true);
                lowerMagnitudeRatioJSON = new JSONStorableFloat("Lower Magnitude Coeff", -0.25f, -1f, 1f, true, true);
                quicknessRatioJSON = new JSONStorableFloat("Quickness Coeff", 2f, 0f, 4f, false, true);
                upperMagnitudeJSON = new JSONStorableFloat("upperMagnitude", 150f, -50f, 150f, false, true);
                targetMagnitudeJSON = new JSONStorableFloat("targetMagnitude", 0.0f, -50f, 50f, false, false);
                
                avatarMobility = new JSONStorableFloat("Avatar Mobility", 2f, 0.5f, 10f, true, true);
                windStrength = new JSONStorableFloat("Wind Strength", 0f, 0.0f, 300f, false, true);
                windTurbulence = new JSONStorableFloat("Wind dir Turbulence", 0.5f, 0.0f, 1.0f, false, true);
                oscillationVariance = new JSONStorableFloat("Oscillation Variance", 1.5f, 1f, 100f, true, true);
                

				RegisterBool(isAvatarJSON);
                //RegisterBool(isGlobalJSON);
                //RegisterStringChooser(receiverJSON);
                //RegisterStringChooser(receiverTargetJSON);

                RegisterBool(autoJSON);
                RegisterBool(oscillatingJSON); 
                RegisterFloat(periodJSON);
                RegisterFloat(quicknessJSON);
                RegisterFloat(upperMagnitudeJSON);
                RegisterFloat(lowerMagnitudeJSON);
                RegisterFloat(lowerMagnitudeRatioJSON);
                RegisterFloat(currentMagnitudeJSON);
                RegisterFloat(windStrength);
                RegisterFloat(windTurbulence);
                RegisterFloat(avatarMobility);
                RegisterFloat(oscillationVariance);
                RegisterFloat(quicknessRatioJSON);
                

                CreateToggle(isAvatarJSON);
                //CreateToggle(isGlobalJSON);
                CreateToggle(autoJSON);
                CreateToggle(oscillatingJSON);
                //CreateScrollablePopup(receiverJSON, true).popupPanelHeight = 1100f;
                //CreateScrollablePopup(receiverTargetJSON, true);
                CreateSlider(windStrength);
                CreateSlider(currentMagnitudeJSON);
                
                CreateSlider(periodJSON);
                //CreateSlider(quicknessJSON);  
                CreateSlider(lowerMagnitudeRatioJSON);
                CreateSlider(lowerMagnitudeJSON);
                CreateSlider(quicknessRatioJSON);
                CreateSlider(quicknessJSON);
                //CreateSlider(upperMagnitudeJSON);
                //CreateSlider(targetMagnitudeJSON);
                CreateSlider(windTurbulence);
                CreateSlider(oscillationVariance);
                //CreateSlider(windDecay);
                CreateSlider(avatarMobility);

                isRadialJSON = new JSONStorableBool("Radial Wind", false, (bool b)=>{
                    SyncIsRadial(b);
                });
                RegisterBool(isRadialJSON);
                CreateToggle(isRadialJSON);
                isBlowingJSON = new JSONStorableBool("Suck/Blow", false, new JSONStorableBool.SetBoolCallback(SyncIsBlowing));
                RegisterBool(isBlowingJSON);
                CreateToggle(isBlowingJSON);
                windDecay = new JSONStorableFloat("Wind Decay", 1f, 1f, 100f, false, true);
                RegisterFloat(windDecay);
                CreateSlider(windDecay);

                
            }
            catch (System.Exception e)
            {
                SuperController.LogError("" + e);
            }
        }

        protected void Update()
        {
            
            if(!loaded)
            {
                updateObjectList();
            }   
            
            if (currentMagnitudeJSON == null)
                return;
            if (autoJSON != null && autoJSON.val)
            {
                timer -= Time.deltaTime;
                if ((double)timer < 0.0)
                {
                    timer = periodJSON.val;
                    //lowerMagnitudeJSON.val=-(windStrength.val/4);
                    lowerMagnitudeJSON.val=windStrength.val*lowerMagnitudeRatioJSON.val;
                    upperMagnitudeJSON.val=windStrength.val;
                    quicknessJSON.val=quicknessRatioJSON.val*windStrength.val/5;
                    //periodJSON.val=0.2f-(windStrength.val/1000);
                    
                    if(oscillatingJSON.val)
                    {
                        if(oscillationVariance.val<1)
                        {
                            oscillationVariance.val=1;
                        }
                        var oscillation_random_temp=Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val)/oscillationVariance.val;
                        if(oscillation)
                        {
                            
                            targetMagnitudeJSON.val=upperMagnitudeJSON.val-oscillation_random_temp;
                            oscillation=false;
                        }
                        else
                        {
                            targetMagnitudeJSON.val=lowerMagnitudeJSON.val+oscillation_random_temp;
                            oscillation=true;
                        }
                    }
                    else
                    {
                        targetMagnitudeJSON.val = Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val);
                    }
                    setWindTurbulence();
                }
                if(windStrength.val==0)
                {
                    currentMagnitudeJSON.val=0;
                }
                else
                {
                currentMagnitudeJSON.val = Mathf.Lerp(currentMagnitudeJSON.val, targetMagnitudeJSON.val, Time.deltaTime * quicknessJSON.val);
                }
            }
            else
            {
                currentMagnitudeJSON.val=windStrength.val;
                //lowerMagnitudeJSON.val=-(windStrength.val/4);
                lowerMagnitudeJSON.val=windStrength.val*lowerMagnitudeRatioJSON.val;
                upperMagnitudeJSON.val=windStrength.val;
                quicknessJSON.val=windStrength.val/5;
                periodJSON.val=0.2f-(windStrength.val/1000);
                if(periodJSON.val<0.03f)
                {
                    periodJSON.val=0.03f;
                }
            }
            setWindTurbulence();
            if(isRadial)
            {
                windRandomY=Random.Range(-windTurbulence.val, windTurbulence.val);
                windRandomX=Random.Range(-windTurbulence.val, windTurbulence.val);
                windRandomZ=Random.Range(-windTurbulence.val, windTurbulence.val);
            }
            Vector3 vector3 = currentMagnitudeJSON.val * newDirection;
            if (isGlobal)
            {

                
            }
            
			if (isAvatar)
			{
                //debug_print_dictionary();
                foreach(KeyValuePair<string,bool> RB_elem in RB_enabled)
                {
                    //Now you can access the key and value both separately from this attachStat as:
                    //SuperController.LogMessage("RB:"+RB_elem.Key+" val:"+RB_elem.Value+"");
                    if(RB_elem.Value)
                    {
                        if(isRadial)
                        {
                            updateControllerRadial(RB_elem.Key, RB_mobility[RB_elem.Key].val);
                        }
                        else
                        {
                            updateController(RB_elem.Key, RB_mobility[RB_elem.Key]);
                        }
                    }
                }
			}

        }

        protected void updateController(string Controller, JSONStorableFloat mobility)
        {
            //SyncReceiverRB(Controller);
            foreach (ForceReceiver fr in SuperController.singleton.GetAtomByUid(Controller).forceReceivers)
            {
                if(fr.name=="object")
                {
                    fr.GetComponent<Rigidbody>().AddForce((currentMagnitudeJSON.val+Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val))/(10*mobility.val*avatarMobility.val) * (newDirection));
                }
            }
            //RB.AddForce((currentMagnitudeJSON.val+Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val))/(10*mobility.val*avatarMobility.val) * (newDirection));
        }

        protected void getDirection(string controller)
        {
            var WindController=containingAtom.GetStorableByID("control").transform;
            var AvatarController=receivingAtom.GetStorableByID(controller).transform;
            if (isBlowing)
            {
                direction=AvatarController.position - (new Vector3(WindController.position.x + windRandomX, WindController.position.y + windRandomY, WindController.position.z + windRandomZ));
            }
            if (!isBlowing)
            {
                direction=(new Vector3(WindController.position.x + windRandomX, WindController.position.y + windRandomY, WindController.position.z + windRandomZ)) - AvatarController.position;
            }
        }

        protected void updateControllerRadial(string Controller, float mobility)
        {
            //SyncReceiverRB(Controller);
            var WindController=containingAtom.GetStorableByID("control").transform;
            var currentController=SuperController.singleton.GetAtomByUid(Controller).GetStorableByID("control").transform;
            var randomValue=Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val);
            var direction=(new Vector3(WindController.position.x + windRandomX, WindController.position.y + windRandomY, WindController.position.z + windRandomZ)) - currentController.position;
            if (isBlowing)
            {
                direction=currentController.position - (new Vector3(WindController.position.x + windRandomX, WindController.position.y + windRandomY, WindController.position.z + windRandomZ));
            }
            var windVariance=Random.Range(lowerMagnitudeJSON.val, upperMagnitudeJSON.val);
            var distanceFactor=direction.magnitude;
            var decayFactor=(distanceFactor-(distanceFactor*windDecay.val)+(avatarMobility.val*5)*mobility);
            if(decayFactor<1)
            {
                decayFactor=1;
            }
            //currentController.position+=(((currentMagnitudeJSON.val+randomValue)/(10000/mobility*avatarMobility.val)) * direction.normalized * windDecay.val/(windDecay.val+direction.magnitude));
            //RB.AddForce((currentMagnitudeJSON.val+windVariance)/decayFactor*direction.normalized);
            foreach (ForceReceiver fr in SuperController.singleton.GetAtomByUid(Controller).forceReceivers)
            {
                if(fr.name=="object")
                {
                    fr.GetComponent<Rigidbody>().AddForce((currentMagnitudeJSON.val+windVariance)/decayFactor*direction.normalized);
                }
            }
        }

        protected void updateObjectList()
        {
            if(loadTimer<=0f)
            {
                getAtomList();
                //RIGIDBODIES
                foreach(KeyValuePair<string,bool> RB_elem in RB_enabled)
                {
                    //Now you can access the key and value both separately from this attachStat as:
                    //SuperController.LogMessage("RB:"+RB_elem.Key+" val:"+RB_elem.Value+"");
                    var tempRB_enab = new JSONStorableBool(RB_elem.Key, RB_elem.Value,(bool enabled)=>
                    {
                        RB_enabled[RB_elem.Key]=enabled;
                        var receivingAtom=SuperController.singleton.GetAtomByUid(RB_elem.Key);
                        
                        foreach (string elem in receivingAtom.GetStorableIDs())
                        {
                            if(elem.Contains("WindControl_Target"))
                            {
                                receivingAtom.GetStorableByID(elem).GetBoolJSONParam("Wind Enabled").val=enabled;
                            }
                        }
                        /*
                        if(enabled)
                        {
                            setControllerPositionState(RB_elem.Key, "Off");
                            setControllerRotationState(RB_elem.Key, "Off");
                        }
                        else
                        {
                            setControllerPositionState(RB_elem.Key, "On");
                            setControllerRotationState(RB_elem.Key, "On");
                        }
                        */
                    });
                    RegisterBool(tempRB_enab);
                    //SuperController.LogMessage("RB:"+tempRB_enab.val+" val:"+RB_elem.Value+"");
                    CreateToggle(tempRB_enab,true);
                    var temp_mobility=RB_mobility[RB_elem.Key].val;
                    RB_mobility[RB_elem.Key] = new JSONStorableFloat(RB_elem.Key+" Mobility", temp_mobility, (float num) => {
                        var receivingAtom=SuperController.singleton.GetAtomByUid(RB_elem.Key);
                        
                        foreach (string elem in receivingAtom.GetStorableIDs())
                        {
                            if(elem.Contains("WindControl_Target"))
                            {
                                receivingAtom.GetStorableByID(elem).GetFloatJSONParam("Mobility").val=num;
                            }
                        }
                    }, 0.1f, 1.0f, true, true);
                    RegisterFloat(RB_mobility[RB_elem.Key]);   
                    CreateSlider(RB_mobility[RB_elem.Key],true);
                    loaded=true;
                }
            }
            else
            {
                loadTimer--;
            }

            
        }

        protected void setControllerPositionState(string controller, string state)
        {
            FreeControllerV3 FCv3 = SuperController.singleton.GetAtomByUid(controller).GetStorableByID("control") as FreeControllerV3;
			//Possible states:
			//On, Off, Following, Hold, Lock, ParentLink, PhysicsLink, Comply

			switch (state)
			{
				case "On":
					FCv3.currentPositionState=FreeControllerV3.PositionState.On;
					break;
				case "Off":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Off;
					break;
				case "Following":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Following;
					break;
				case "Hold":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Hold;
					break;
				case "Lock":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Lock;
					break;
				case "ParentLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.ParentLink;
					break;
				case "PhysicsLink":
					FCv3.currentPositionState=FreeControllerV3.PositionState.PhysicsLink;
					break;
				case "Comply":
					FCv3.currentPositionState=FreeControllerV3.PositionState.Comply;
					break;

				default:
					SuperController.LogError("setControllerPositionState: Unrecognized position status");
					break;
			}
        }

		protected void setControllerRotationState(string controller, string state)
        {
            FreeControllerV3 FCv3 = SuperController.singleton.GetAtomByUid(controller).GetStorableByID("control") as FreeControllerV3;
			//Possible states:
			//On, Off, Following, Hold, Lock, ParentLink, PhysicsLink, Comply

			switch (state)
			{
				case "On":
					FCv3.currentRotationState=FreeControllerV3.RotationState.On;
					break;
				case "Off":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Off;
					break;
				case "Following":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Following;
					break;
				case "Hold":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Hold;
					break;
				case "Lock":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Lock;
					break;
				case "ParentLink":
					FCv3.currentRotationState=FreeControllerV3.RotationState.ParentLink;
					break;
				case "PhysicsLink":
					FCv3.currentRotationState=FreeControllerV3.RotationState.PhysicsLink;
					break;
				case "Comply":
					FCv3.currentRotationState=FreeControllerV3.RotationState.Comply;
					break;

				default:
					SuperController.LogError("setControllerRotationState: Unrecognized Rotation status");
					break;
			}
        }


        protected void setWindTurbulence()
        {
            var windRandomY=Random.Range(0, windTurbulence.val);
            var windRandomX=Random.Range(-windTurbulence.val, windTurbulence.val);
            var windRandomZ=Random.Range(-windTurbulence.val, windTurbulence.val);
            newDirection = new Vector3(containingAtom.GetStorableByID("control").transform.forward.x + windRandomX, containingAtom.GetStorableByID("control").transform.forward.y , containingAtom.GetStorableByID("control").transform.forward.z + windRandomZ).normalized;
        }

        protected void OnDestroy()
        {

        }
    }

}
